# Client for EAIS grpc service
import grpc
import eais_pb2
import eais_pb2_grpc
from google.protobuf.any_pb2 import Any
import pandas as pd
import os
import joblib
from utils import get_file_chunks, save_chunks_to_file, data_dir, models_dir, analysis_dir
# directories

port_addrs = 'localhost:8061'



# create a gRPC channel + stub
channel = grpc.insecure_channel(port_addrs)

# create a stub client
stub = eais_pb2_grpc.ShapAnalyticsStub(channel)

# load data and models
# dataset = pd.read_csv(os.path.join(data_dir, 'X.csv'))
# model = joblib.load(os.path.join(models_dir, 'xgb.joblib'))


if os.path.isfile(os.path.join(data_dir, 'X.csv')):
    print("File exists")

# Push data and model to server
data_chunk_generator = get_file_chunks(os.path.join(data_dir, 'X.csv'))
responseData = stub.uploadData(data_chunk_generator)
print(responseData)
#
model_chunker = get_file_chunks(os.path.join(models_dir, 'xgb.joblib'))
responseModel = stub.uploadModel(model_chunker)
print(responseModel)

# Execute analysis
responseAnalysis = stub.shapely_analysis(eais_pb2.Analysis(explainer='shap.explainers._tree',
                                                           model='xgb.joblib',
                                                           dataset='X.csv'))
print(responseAnalysis)
save_chunks_to_file(responseAnalysis, os.path.join(analysis_dir, "new_analysis.tar.gz"))
# req_construct = {
#     "explainer": 'shap.explainers._tree',
#     "model": model,
#     "dataset": dataset,
# }
#
# requestAnalyse = eais_pb2.Data(**req_construct)
# print(requestAnalyse)