import grpc
from concurrent import futures
import time
# import the generated classes :
import eais_pb2
import eais_pb2_grpc
import uuid
# import the function we made :
import shapely_analytics as sa
import os
from utils import save_chunks_to_file, get_file_chunks, data_dir, models_dir, analysis_dir
port = 8061
# create a class to define the server functions, derived from


class ShapAnalyticsServicer(eais_pb2_grpc.ShapAnalyticsServicer):
    def uploadModel(self, request_iterator, context):
        model_name = f"model_{str(uuid.uuid4())[:8]}.joblib"
        save_chunks_to_file(request_iterator, os.path.join(models_dir, model_name))
        return eais_pb2.Response(status=model_name)

    def uploadData(self, request_iterator, context):
        data_name = f"X_{str(uuid.uuid4())[:8]}.csv"
        save_chunks_to_file(request_iterator, os.path.join(data_dir, data_name))
        return eais_pb2.Response(status=data_name)

    def shapely_analysis(self, request, context):
        # define the buffer of the response :
        # get the value of the response by calling the desired function :
        analysis_file = sa.shap_analytics(request)
        return get_file_chunks(analysis_file)

# create a gRPC server
server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
eais_pb2_grpc.add_ShapAnalyticsServicer_to_server(ShapAnalyticsServicer(), server)
print("Starting EAIS server. Listening on port " + str(port))
server.add_insecure_port('[::]:' + str(port))
server.start()
try:
    while True:
        time.sleep(86400)
except KeyboardInterrupt:
    server.stop(0)