import shap
import pandas as pd
import numpy as np
import tarfile
from importlib import import_module
from joblib import load, dump
import os
from utils import data_dir, models_dir, analysis_dir
import uuid
import tarfile

def shap_analytics(input_data):
    # Load data and models
    model = load(os.path.join(models_dir, input_data.model))
    X = pd.read_csv(os.path.join(data_dir, input_data.dataset))
    # Instantiate explainer instance
    explainer_name = input_data.explainer
    mod = import_module(f"{explainer_name}")
    # Run analysis
    print("Starting analysis")
    explainer = mod.Tree(model)
    print("Explainer instantiated")
    shap_values = explainer.shap_values(X.to_numpy())
    print("Finished analysis")
    #Save explainer and shap_values
    explainer_uuid = str(uuid.uuid4())[:8]
    explainer_file = os.path.join(analysis_dir, f'explainer_{explainer_uuid}.sav')
    dump(explainer, explainer_file)
    shap_values_file = os.path.join(analysis_dir, f"shap_values_{explainer_uuid}.sav")
    dump(shap_values, shap_values_file)
    analysis_file = os.path.join(analysis_dir, f"shap_analysis_{explainer_uuid}.tar.gz")
    with tarfile.open(analysis_file, 'w') as tw:
        for filename in [explainer_file, shap_values_file]:
            tw.add(filename)
    return analysis_file







# import pathlib
# import tarfile
#
# with tarfile.open('combined.tar.gz', 'w') as tw:
#     for filename in pathlib.Path('./target_gz').glob('*.gz'):
#         print(filename)
#         tw.add(filename)